/*
 * GAPI implementation for HP Jornada 820
 *
 * this software is copyright (c) 2003 Hasjim Williams
 * this software is distributed under GNU Lesser General Public License
 * see LICENSE.txt for details 
 *
 * This code is based off the GAPI implementation for the Casio E100/E105
 */

#define STRICT
#include <windows.h>
#include "stdafx.h"

#define GXDLL_EXPORTS
#include <gx.h>

/***********************************************************************************************
CONSTANTS for the JORNADA 820 LCD
***********************************************************************************************/
#define SA1100_FRAMEBUFFER  0xC0003000

#define SA1100_LCD_WIDTH    640
#define SA1100_LCD_HEIGHT   480
#define SA1100_LCD_BPP      8
#define SA1100_LCD_FFFORMAT kfPalette

#define SA1100_LCD_XPITCH   (SA1100_LCD_BPP / 8)
#define SA1100_LCD_YPITCH	SA1100_LCD_WIDTH * SA1100_LCD_XPITCH

/* 
VirtualCopy is the magic function which allows booting Linux on Windows CE 
devices.

  http://msdn.microsoft.com/library/default.asp?url=/library/en-us/wceoem/htm/krnl_f_33.asp
  
	there is one KB article about it
	http://support.microsoft.com/support/kb/articles/Q213/3/59.ASP
	  
		parameters are:
		LPVOID pMemoryBlock
		LPVOID pPhysicalAddress
		DWORD dwBlockSize
		DWORD dwProtectionFlags
		
		  it seems there are two ways to map memory
		  if physical address is less than 512MB it should be ORed with 0x80000000L
		  and ProtectionFlags should be PAGE_READWRITE | PAGE_NOCACHE
		  else
		  physical address has to be shifted right by 8 abd
		  ProtectionFlags should be PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL
*/   

extern "C" {
	__declspec(dllimport) BOOL VirtualCopy(LPVOID, LPVOID, DWORD, DWORD);
};

/***********************************************************************************************
GLOBALS
***********************************************************************************************/

static DWORD g_dwFlags = 0;		
static char *g_pMem = 0;	// Pointer to the framebuffer's virtual address
static HWND	g_hWnd = 0;

static unsigned int g_uInputOpened = 0;

/***********************************************************************************************

***********************************************************************************************/
BOOL APIENTRY 
DllMain(HANDLE hModule, 
        DWORD  ul_reason_for_call, 
        LPVOID lpReserved)
{
	SYSTEM_INFO stSI;
	OSVERSIONINFO stOI;
	HKEY hKey;
	DWORD dwType; 
	TCHAR aszName[256];
	DWORD dwSize;
	
	stOI.dwOSVersionInfoSize = sizeof(stOI);
	
	GetSystemInfo(&stSI);
	GetVersionEx(&stOI);
	
	if (!(
		stSI.dwProcessorType == PROCESSOR_STRONGARM
		&& stSI.wProcessorArchitecture == 0x5 // 0x5 == ??? 
		&& stOI.dwMajorVersion == 2
		&& stOI.dwMinorVersion == 11
		))
		return FALSE;
	
	if (ERROR_SUCCESS != RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("Platform"), 0, 0, &hKey))
		return FALSE;
	dwSize = 256 * sizeof(TCHAR);
	if (ERROR_SUCCESS != RegQueryValueEx(hKey, TEXT("Name"), 0, &dwType, (LPBYTE)aszName, &dwSize))
	{
		RegCloseKey(hKey);
		return FALSE;
	}
	RegCloseKey(hKey);
	/* The Jornada 820's Platform Name is "ThunderBird" ??? */
	if (wcscmp(aszName, TEXT("ThunderBird")) != 0)
		return FALSE;

	return TRUE;
}

/***********************************************************************************************

***********************************************************************************************/

GXDLL_API int GXOpenDisplay(HWND hWnd, DWORD dwFlags) {
	//	DWORD dwPageSize;
	DWORD dwSize;
	
	
	if (!hWnd)
		hWnd = GetActiveWindow();
	g_dwFlags = dwFlags;
	g_hWnd = hWnd;
	
	g_uInputOpened = 0;
	
	dwSize = SA1100_LCD_WIDTH * SA1100_LCD_HEIGHT; // 307200 bytes for 640 x 480
	
	g_pMem = (char *)VirtualAlloc((LPVOID)0, (DWORD) dwSize, (DWORD) MEM_RESERVE, (DWORD) PAGE_NOACCESS);
	if (!g_pMem)
		return 0;
	
		/* it seems there are two ways to map memory
		if physical address is less than 512MB it should be ORed with 0x80000000L
		and ProtectionFlags should be PAGE_READWRITE | PAGE_NOCACHE
		else
		physical address has to be shifted right by 8 and
	ProtectionFlags should be PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL */
	
	VirtualCopy((LPVOID)g_pMem, 
		(LPVOID)((SA1100_FRAMEBUFFER >> 8)), 
		(DWORD) dwSize, 
		(DWORD) PAGE_READWRITE | PAGE_NOCACHE | PAGE_PHYSICAL);
	
	// if (g_dwFlags & GX_FULLSCREEN)
	//    TaskbarHide();
	
	return TRUE;
}

extern "C" int 
compat_GXOpenDisplay(HWND hWnd, DWORD dwFlags);

int 
compat_GXOpenDisplay(HWND hWnd, DWORD dwFlags)
{
	return GXOpenDisplay(hWnd, dwFlags) ;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXCloseDisplay() {
	if (!g_pMem)
		return 0;
	
	//	if (g_uTaskbarHidden)
	//		TaskbarShow();
	//	if (g_uTaskbarSubclassed)
	//		TaskbarRestore();
	
	VirtualFree((LPVOID)g_pMem, 0, MEM_RELEASE);
	g_pMem = 0;
	return 1;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API void* GXBeginDraw() {
	return (void *)g_pMem;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXEndDraw() {
	return 1;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXOpenInput() {
	return 1;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXCloseInput() {
	return 1;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API GXDisplayProperties GXGetDisplayProperties() {
	struct GXDisplayProperties dp;
	dp.cxWidth = SA1100_LCD_WIDTH;
	dp.cyHeight = SA1100_LCD_HEIGHT;
	dp.cBPP = SA1100_LCD_BPP;
	dp.cbxPitch = SA1100_LCD_XPITCH;
	dp.cbyPitch = SA1100_LCD_YPITCH;
	dp.ffFormat = SA1100_LCD_FFFORMAT;
	return dp;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API GXKeyList GXGetDefaultKeys(int iOptions) {
	struct GXKeyList kl;
	
	kl.vkLeft  = VK_LEFT;
	kl.vkRight = VK_RIGHT;
	kl.vkUp    = VK_UP;
	kl.vkDown  = VK_DOWN;
	kl.vkStart = VK_RETURN;
	kl.vkA     = 'Z';
	kl.vkB     = 'X';
	kl.vkC     = 'C';
	
	return kl;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXSuspend() {
	return 0;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXResume() {
	return 0;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API int GXSetViewport(DWORD dwTop, DWORD dwHeight, DWORD dwReserved1, DWORD dwReserved2) {
	return 0;
}

/***********************************************************************************************

***********************************************************************************************/
GXDLL_API BOOL GXIsDisplayDRAMBuffer() {
	return TRUE;
}
